==> nanome-jax-1.3.16.tar.gz: nanome-jax-1.3.16/src/nanocompare/xgboost/xgboost_predict.py <==
#!/usr/bin/env python3
# @Author   : Yang Liu
# @FileName : xgboost_predict.py
# @Software : NANOME project
# @Organization : JAX Li Lab
# @Website  : https://github.com/TheJacksonLaboratory/nanome

import argparse
from functools import reduce

import joblib
import pandas as pd

from nanocompare.eval_common import load_tool_read_level_unified_as_df
from nanocompare.global_config import set_log_debug_level, set_log_info_level, logger
from nanocompare.global_settings import nanome_model_dict, CHUNKSIZE, NANOME_VERSION


def parse_arguments():
    parser = argparse.ArgumentParser(prog='xgboost_predict (NANOME)', description='XGBoost predict for data')
    parser.add_argument('-v', '--version', action='version', version=f'%(prog)s v{NANOME_VERSION}')
    parser.add_argument('-i', type=str, required=True,
                        help='input data for predicting')
    parser.add_argument('-m', type=str, required=True,
                        help=f'model file, existing model list: {",".join(list(nanome_model_dict.keys()))}')
    parser.add_argument('--dsname', type=str, required=True,
                        help='dataset name')
    parser.add_argument('--random-state', type=int, default=42,
                        help='random state, default is 42')
    parser.add_argument('--processors', type=int, default=8,
                        help='num of processors, default is 8')
    parser.add_argument('--chunksize', type=int, default=CHUNKSIZE,
                        help=f'chunk size for load large data, default is {CHUNKSIZE}')
    parser.add_argument('-o', type=str, required=True,
                        help='output file name')
    parser.add_argument('--contain-na', help="if make prediction on NA values", action='store_true')
    parser.add_argument('--tsv-input', help="if input is tsv for tools' read-level format, or else is combined input",
                        action='store_true')
    parser.add_argument('--chrs', nargs='+', help='chromosomes used', default=[])
    parser.add_argument('--verbose', help="if output verbose info", action='store_true')
    args = parser.parse_args()
    return args


if __name__ == '__main__':
    args = parse_arguments()
    if args.verbose:
        set_log_debug_level()
    else:
        set_log_info_level()
    logger.debug(f"args={args}")

    if args.m in nanome_model_dict:
        infn = nanome_model_dict[args.m]
    else:
        infn = args.m
    logger.debug(f"Model file: {infn}")
    xgboost_cls = joblib.load(infn)
    logger.debug(f"Model info: xgboost_cls={xgboost_cls}, best_params={xgboost_cls.best_params_}")

    if args.tsv_input:
        df_tsvfile = pd.read_csv(args.i, header=None, sep='\t')
        tool_list = list(df_tsvfile[0])
        dflist = []
        for index, row in df_tsvfile.iterrows():
            df = load_tool_read_level_unified_as_df(row[1], toolname=row[0], filterChrs=args.chrs,
                                                    chunksize=args.chunksize)
            dflist.append(df)
        if args.contain_na:
            datadf = reduce(
                lambda left, right: pd.merge(left, right, how='outer', on=["ID", "Chr", "Pos", "Strand"]),
                dflist)
        else:
            datadf = reduce(
                lambda left, right: pd.merge(left, right, how='inner', on=["ID", "Chr", "Pos", "Strand"]),
                dflist)
        # release memory
        dflist = None
        datadf.drop_duplicates(subset=["ID", "Chr", "Pos", "Strand"], inplace=True)
        datadf.reset_index(inplace=True, drop=True)

        if len(datadf) <= 0:
            raise Exception(f"The combined results are empty, for tool_list={tool_list}, fn_list={df_tsvfile[1]}")

        logger.debug(f"tool_list={tool_list}")
        logger.debug(f"datadf={datadf}")
    else:  # combined input as default input
        if len(args.chrs) >= 1:
            iter_df = pd.read_csv(args.i, header=0, index_col=False, sep="\t", iterator=True,
                                  chunksize=args.chunksize)
            datadf = pd.concat([chunk[chunk['Chr'].isin(args.chrs)] for chunk in iter_df])
        else:
            datadf = pd.read_csv(args.i, header=0, sep='\t', index_col=False)
        tool_list = list(datadf.columns[4:-2])
        if not args.contain_na:  ## remove NAs
            datadf.dropna(subset=list(datadf.columns[0:-2]), inplace=True)
        datadf.drop_duplicates(subset=["ID", "Chr", "Pos", "Strand"], inplace=True)
        datadf.dropna(subset=tool_list, inplace=True, how='all')
        datadf.reset_index(inplace=True, drop=True)
        datadf = datadf.loc[:, list(datadf.columns[0:-2])]

        logger.debug(f"tool_list={tool_list}")
        logger.debug(f"datadf={datadf}")

    ## Output read-level, site-level distributions
    logger.debug(f"Read stats: total={len(datadf):,}")

    sitedf = datadf[["Chr", "Pos", "Strand"]].drop_duplicates()
    logger.debug(f"Site stats: total={len(sitedf):,}")
    sitedf = None

    predX = datadf.loc[:, tool_list]
    prediction = pd.DataFrame(xgboost_cls.predict(predX))
    prediction.rename(columns={0: "Prediction"}, inplace=True)

    prediction_prob = pd.DataFrame(xgboost_cls.predict_proba(predX))[[1]]
    prediction_prob.rename(columns={1: "Prob_methylation"}, inplace=True)

    nanome_df = pd.concat([datadf, prediction, prediction_prob], axis=1)
    nanome_df = nanome_df[
        ['ID', 'Chr', 'Pos', 'Strand'] + tool_list + ["Prediction", "Prob_methylation"]]
    logger.debug(f"nanome_df={nanome_df}")

    ## APL.nanopolish.methylation_calls.combine.tsv.gz
    nanome_df.to_csv(args.o, sep='\t', index=False)
    logger.info(f"save to {args.o}")

==> nanome-jax-1.3.18.tar.gz: nanome-jax-1.3.18/src/nanocompare/xgboost/xgboost_predict.py <==
#!/usr/bin/env python3
# @Author   : Yang Liu
# @FileName : xgboost_predict.py
# @Software : NANOME project
# @Organization : JAX Li Lab
# @Website  : https://github.com/TheJacksonLaboratory/nanome

import argparse
from functools import reduce

import joblib
import pandas as pd

from nanocompare.eval_common import load_tool_read_level_unified_as_df
from nanocompare.global_config import set_log_debug_level, set_log_info_level, logger
from nanocompare.global_settings import nanome_model_dict, CHUNKSIZE, NANOME_VERSION


def parse_arguments():
    parser = argparse.ArgumentParser(prog='xgboost_predict (NANOME)', description='XGBoost predict for data')
    parser.add_argument('-v', '--version', action='version', version=f'%(prog)s v{NANOME_VERSION}')
    parser.add_argument('-i', type=str, required=True,
                        help='input data for predicting')
    parser.add_argument('-m', type=str, required=True,
                        help=f'model file, existing model list: {",".join(list(nanome_model_dict.keys()))}')
    parser.add_argument('--dsname', type=str, required=True,
                        help='dataset name')
    parser.add_argument('--random-state', type=int, default=42,
                        help='random state, default is 42')
    parser.add_argument('--processors', type=int, default=8,
                        help='num of processors, default is 8')
    parser.add_argument('--chunksize', type=int, default=CHUNKSIZE,
                        help=f'chunk size for load large data, default is {CHUNKSIZE}')
    parser.add_argument('-o', type=str, required=True,
                        help='output file name')
    parser.add_argument('--contain-na', help="if make prediction on NA values", action='store_true')
    parser.add_argument('--tsv-input', help="if input is tsv for tools' read-level format, or else is combined input",
                        action='store_true')
    parser.add_argument('--chrs', nargs='+', help='chromosomes used', default=[])
    parser.add_argument('--verbose', help="if output verbose info", action='store_true')
    args = parser.parse_args()
    return args


if __name__ == '__main__':
    args = parse_arguments()
    if args.verbose:
        set_log_debug_level()
    else:
        set_log_info_level()
    logger.debug(f"args={args}")

    if args.m in nanome_model_dict:
        infn = nanome_model_dict[args.m]
    else:
        infn = args.m
    logger.debug(f"Model file: {infn}")
    xgboost_cls = joblib.load(infn)
    try:
        logger.debug(f"Model info: xgboost_cls={xgboost_cls}")
        logger.debug(f"best_params={xgboost_cls.best_params_}")
    except:
        logger.debug(f"WARNNING: print params encounter problem")

    if args.tsv_input:
        df_tsvfile = pd.read_csv(args.i, header=None, sep='\t')
        tool_list = list(df_tsvfile[0])
        dflist = []
        for index, row in df_tsvfile.iterrows():
            df = load_tool_read_level_unified_as_df(row[1], toolname=row[0], filterChrs=args.chrs,
                                                    chunksize=args.chunksize)
            dflist.append(df)
        if args.contain_na:
            datadf = reduce(
                lambda left, right: pd.merge(left, right, how='outer', on=["ID", "Chr", "Pos", "Strand"]),
                dflist)
        else:
            datadf = reduce(
                lambda left, right: pd.merge(left, right, how='inner', on=["ID", "Chr", "Pos", "Strand"]),
                dflist)
        # release memory
        dflist = None
        datadf.drop_duplicates(subset=["ID", "Chr", "Pos", "Strand"], inplace=True)
        datadf.reset_index(inplace=True, drop=True)

        if len(datadf) <= 0:
            raise Exception(f"The combined results are empty, for tool_list={tool_list}, fn_list={df_tsvfile[1]}")

        logger.debug(f"tool_list={tool_list}")
        logger.debug(f"datadf={datadf}")
    else:  # combined input as default input
        if len(args.chrs) >= 1:
            iter_df = pd.read_csv(args.i, header=0, index_col=False, sep="\t", iterator=True,
                                  chunksize=args.chunksize)
            datadf = pd.concat([chunk[chunk['Chr'].isin(args.chrs)] for chunk in iter_df])
        else:
            datadf = pd.read_csv(args.i, header=0, sep='\t', index_col=False)
        tool_list = list(datadf.columns[4:-2])
        if not args.contain_na:  ## remove NAs
            datadf.dropna(subset=list(datadf.columns[0:-2]), inplace=True)
        datadf.drop_duplicates(subset=["ID", "Chr", "Pos", "Strand"], inplace=True)
        datadf.dropna(subset=tool_list, inplace=True, how='all')
        datadf.reset_index(inplace=True, drop=True)
        datadf = datadf.loc[:, list(datadf.columns[0:-2])]

        logger.debug(f"tool_list={tool_list}")
        logger.debug(f"datadf={datadf}")

    ## Output read-level, site-level distributions
    logger.debug(f"Read stats: total={len(datadf):,}")

    sitedf = datadf[["Chr", "Pos", "Strand"]].drop_duplicates()
    logger.debug(f"Site stats: total={len(sitedf):,}")
    sitedf = None

    logger.debug(f"Start predict by XGBoost......")
    predX = datadf.loc[:, tool_list]
    prediction = pd.DataFrame(xgboost_cls.predict(predX))
    prediction.rename(columns={0: "Prediction"}, inplace=True)

    prediction_prob = pd.DataFrame(xgboost_cls.predict_proba(predX))[[1]]
    prediction_prob.rename(columns={1: "Prob_methylation"}, inplace=True)

    nanome_df = pd.concat([datadf, prediction, prediction_prob], axis=1)
    nanome_df = nanome_df[
        ['ID', 'Chr', 'Pos', 'Strand'] + tool_list + ["Prediction", "Prob_methylation"]]
    logger.debug(f"nanome_df={nanome_df}")

    ## APL.nanopolish.methylation_calls.combine.tsv.gz
    nanome_df.to_csv(args.o, sep='\t', index=False)
    logger.info(f"save to {args.o}")
    logger.info("Done for XGBoost predict")
